
# Procedure d'exploitation

## Démarrage de la solution 

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml start_vitamui.yml
~~~

## Arrêt de la solution

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml stop_vitamui.yml
~~~

## Upgrade de la solution

Actuellement, un upgrade de solution s'apparente à une installation de la solution.

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml vitamui.yml
~~~

Les services VitamUI sont stateless et vont être remplacés par une nouvelle version lors de 
l'opération précédente.

Les scripts de base de données MongoDb sont versionnés via un changelog interne. Un même
script ne peut pas être joué 2 fois. Les nouveaux scripts vont être intégrés et les anciens
scripts ou doublons de scripts ne vont pas être pris en compte. 


## Sauvegarde de la solution

La base de données Mongodb utilisée par VitamUI peut être sauvegardée de 2 manières:

  1. via un playbook de sauvegarde (mongo_backup.yml)
  2. via un dump (commande mongodump)

### Sauvegarde via playbook (1)

Au préalable il est nécessaire de valoriser correctement les variables:

~~~console
mongo_dump_folder: /backup/mongod/
mongo_backup_reinstall:
  - db: "iam"
    collections: ["customers","externalParameters","groups","owners","profiles",
    "sequences","subrogations","tenants","users","providers"]
  - db: "admin"
    collections: []
~~~

La structure mongo_backup_reinstall va permettre de sauvegarder la base entière si elle est renseignée de la sorte:

~~~console
- db: "admin"
    collections: []
~~~

Et les collections uniquement lorsque mentionné comme suit:

~~~console
  - db: "iam"
    collections: ["customers","externalParameters","groups","owners","profiles",
    "sequences","subrogations","tenants","users","providers"]
~~~

Le résultat du backup pourra se retrouver dans

~~~console
mongo_dump_folder: /backup/mongod/
~~~

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml mongo_backup.yml
~~~


### Sauvegarde via la commande mongodump (2)

Se rendre sur la machine (vm) hébergeant l'instance mongodb de VitamUI
et exécuter la commande suivante:

~~~console
mongodump --host "ip_machine:27017" -u [user] -p [mdp] -v --gzip 
--out="/backup/mongod/`date \+\%Y\%m\%d_\%s`"
~~~

## Restoration de la solution

De la même manière, la base de données Mongodb utilisée par VitamUI peut être 
sauvegardée de 2 manières:

  1. via un playbook de sauvegarde (mongo_backup.yml)
  2. via un dump (commande mongodump)

### Restoration via playbook (1)

La restoration va chercher le backup présent dans le dossier que renseigne la variable "mongo_dump_folder".

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml 
--extra-vars \"keep_mongodb_conf=True\" mongo_backup.yml
~~~

### Restoration via la commande mongodump (2)

Se rendre sur la machine (vm) hébergeant l'instance mongodb de VitamUI
et exécuter la commande suivante:

~~~console
mongorestore --host [ip machinie] --port 27017 --username [user] 
--password [mdp] "/backup/mongod/[nom_backup] --drop --gzip --maintainInsertionOrder
~~~


> Attention ! Certaines informations Vitam sont sauvegardées dans VitamUI.  


## Changement de PKI.

Les certificats utilisés par Vitam et VitamUI peuvent être amenés à être changés
en raison de sécurité ou simplement de par leur expiration.

### Certificat Vitamui client expiré dans Vitam

Le certificat client VitamUI.crt est utilisé par Vitam afin que l'application VitamUi
accède aux informations de Vitam. Son échange se fait par les playbooks Vitam
remove_context.yml et add_context.yml dans les sources de Vitam.

Renseigner le fichier postinstall_param.yml avec les
informations du certificat actuel et lancer la commande suppression suivante:

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml remove_context.yml
~~~

Se connecter à l'Ihm_demo de Vitam pour vérifier la bonne suppression du context vitamui_context
dans les contextes applicatifs Vitam.

Une fois cette opération réalisée, il faut :
_Régénérer ou échanger le certificat vitamui.crt afin qu'il soit valide.
_Modifier les informations du fichier 
postinstall_param.yml avec le nom du nouveau certificat et lancer la commande suivante:

~~~console
ansible-playbook  -i environments/<hostfile environnement> --vault-password-file
vault_pass.txt --extra-vars=@./environments/vitamui_extra_vars.yml add_context.yml
~~~

Vérifier alors dans les contextes applicatifs Vitam que le contexte vitamui_context est de nouveau 
présent (rattaché de manière transparente au nouveau certificat).


### Certificats expirés dans les services VitamUI

Regénérer la PKI complète VitamUI et Vitamui. 

Rédéployer la PKI pour les 2 solutions.

Les commandes sont précisées dans le dossier d'installation VitamUI.


## Analyse des problèmes VitamUI

### Logs

Les traces applicatives ou techniques de VitamUI se trouvent sur les machines hébergeant l'application
dans /vitamui/log/nom_service/ pour le service considéré.

Les logs sont écrits via le service rsyslog installé sur toutes les machines de la solution.

### MongoDb

Se connecter à la vm où est hébergé l'instance mongo.

Se connecter à la base de données via l'utilitaire mongo.

Exemple de commande:

~~~console
mongo --host <ip machine> --port 27017 admin --username=<user> --password=<password>
~~~